@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 2009 - 2011, Lefteris Zafiris
+ * Copyright (C) 2009 - 2015, Lefteris Zafiris
  *
  * Lefteris Zafiris <zaf.000@gmail.com>
  *
@@ -35,16 +35,15 @@
 
 ASTERISK_FILE_VERSION(__FILE__, "$Revision: 00 $")
 #include <stdio.h>
+#include <stdlib.h>
 #include <espeak/speak_lib.h>
 #include <sndfile.h>
 #include <samplerate.h>
-#include "asterisk/file.h"
+#include "asterisk/app.h"
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
 #include "asterisk/config.h"
-#include "asterisk/app.h"
 #include "asterisk/utils.h"
-#include "asterisk/strings.h"
 
 #define AST_MODULE "eSpeak"
 #define ESPEAK_CONFIG "espeak.conf"
@@ -63,9 +62,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 00 $")
 /* espeak buffer size in msec */
 #define ESPK_BUFFER 2000
 
-static char *app = "eSpeak";
-static char *synopsis = "Say text to the user, using eSpeak speech synthesizer.";
-static char *descrip =
+static const char *app = "eSpeak";
+static const char *synopsis = "Say text to the user, using eSpeak speech synthesizer.";
+static const char *descrip =
 	"  eSpeak(text[,intkeys,language]):  This will invoke the eSpeak TTS engine,\n"
 	"send a text string, get back the resulting waveform and play it to\n"
 	"the user, allowing any given interrupt keys to immediately terminate\n"
@@ -83,7 +82,7 @@ static int pitch;
 static int capind;
 static const char *def_voice;
 
-static int read_config(void)
+static int read_config(const char *espeak_conf)
 {
 	const char *temp;
 	/* Setting defaut config values */
@@ -97,41 +96,62 @@ static int read_config(void)
 	capind = DEF_CAPIND;
 	def_voice = DEF_VOICE;
 
-	cfg = ast_config_load(ESPEAK_CONFIG, config_flags);
+	cfg = ast_config_load(espeak_conf, config_flags);
 
 	if (!cfg || cfg == CONFIG_STATUS_FILEINVALID) {
 		ast_log(LOG_WARNING,
-				"eSpeak: Unable to read confing file %s. Using default settings\n",
-				ESPEAK_CONFIG);
+				"eSpeak: Unable to read confing file %s. Using default settings\n", espeak_conf);
 	} else {
 		if ((temp = ast_variable_retrieve(cfg, "general", "usecache")))
 			usecache = ast_true(temp);
-
 		if ((temp = ast_variable_retrieve(cfg, "general", "cachedir")))
 			cachedir = temp;
-
-		if ((temp = ast_variable_retrieve(cfg, "general", "samplerate")))
-			target_sample_rate = atoi(temp);
-
-		if ((temp = ast_variable_retrieve(cfg, "voice", "speed")))
-			speed = atoi(temp);
-
-		if ((temp = ast_variable_retrieve(cfg, "voice", "wordgap")))
-			wordgap = atoi(temp);
-
-		if ((temp = ast_variable_retrieve(cfg, "voice", "volume")))
-			volume = atoi(temp);
-
-		if ((temp = ast_variable_retrieve(cfg, "voice", "pitch")))
-			pitch = atoi(temp);
-
-		if ((temp = ast_variable_retrieve(cfg, "voice", "capind")))
-			capind = atoi(temp);
-
+		if ((temp = ast_variable_retrieve(cfg, "general", "samplerate"))) {
+			target_sample_rate = (int) strtol(temp, NULL, 10);
+			if (errno == ERANGE) {
+				ast_log(LOG_WARNING, "eSpeak: Error reading samplerate from config file\n");
+				target_sample_rate = DEF_RATE;
+			}
+		}
+		if ((temp = ast_variable_retrieve(cfg, "voice", "speed"))) {
+			speed = (int) strtol(temp, NULL, 10);
+			if (errno == ERANGE) {
+				ast_log(LOG_WARNING, "eSpeak: Error reading voice speed from config file\n");
+				speed = DEF_SPEED;
+			}
+		}
+		if ((temp = ast_variable_retrieve(cfg, "voice", "wordgap"))) {
+			wordgap = (int) strtol(temp, NULL, 10);
+			if (errno == ERANGE) {
+				ast_log(LOG_WARNING, "eSpeak: Error reading wordgap from config file\n");
+				wordgap = DEF_WORDGAP;
+			}
+		}
+		if ((temp = ast_variable_retrieve(cfg, "voice", "volume"))) {
+			volume = (int) strtol(temp, NULL, 10);
+			if (errno == ERANGE) {
+				ast_log(LOG_WARNING, "eSpeak: Error reading volume from config file\n");
+				volume = DEF_VOLUME;
+			}
+		}
+		if ((temp = ast_variable_retrieve(cfg, "voice", "pitch"))) {
+			pitch = (int) strtol(temp, NULL, 10);
+			if (errno == ERANGE) {
+				ast_log(LOG_WARNING, "eSpeak: Error reading pitch from config file\n");
+				pitch = DEF_PITCH;
+			}
+		}
+		if ((temp = ast_variable_retrieve(cfg, "voice", "capind"))) {
+			capind = (int) strtol(temp, NULL, 10);
+			if (errno == ERANGE) {
+				ast_log(LOG_WARNING, "eSpeak: Error reading capind from config file\n");
+				capind = DEF_CAPIND;
+			}
+		}
 		if ((temp = ast_variable_retrieve(cfg, "voice", "voice")))
 			def_voice = temp;
 	}
-	
+
 	if (target_sample_rate != 8000 && target_sample_rate != 16000) {
 		ast_log(LOG_WARNING,
 				"eSpeak: Unsupported sample rate: %lf. Falling back to %d\n",
@@ -165,8 +185,7 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 	float *src_buff, *dst_buff;
 	char *mydata;
 	int writecache = 0;
-	char MD5_name[33] = "";
-	char cachefile[MAXLEN] = "";
+	char cachefile[MAXLEN];
 	char raw_name[17] = "/tmp/espk_XXXXXX";
 	char slin_name[23];
 	int sample_rate;
@@ -181,7 +200,6 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 		ast_log(LOG_ERROR, "eSpeak requires arguments (text and options)\n");
 		return -1;
 	}
-
 	mydata = ast_strdupa(data);
 	AST_STANDARD_APP_ARGS(args, mydata);
 
@@ -195,7 +213,6 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 	}
 
 	args.text = ast_strip_quoted(args.text, "\"", "\"");
-
 	if (ast_strlen_zero(args.text)) {
 		ast_log(LOG_WARNING, "eSpeak: No text passed for synthesis.\n");
 		return res;
@@ -207,6 +224,7 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 
 	/*Cache mechanism */
 	if (usecache) {
+		char MD5_name[33];
 		ast_md5_hash(MD5_name, args.text);
 		if (strlen(cachedir) + strlen(MD5_name) + 6 <= MAXLEN) {
 			ast_debug(1, "eSpeak: Activating cache mechanism...\n");
@@ -216,12 +234,12 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 				writecache = 1;
 			} else {
 				ast_debug(1, "eSpeak: Cache file exists.\n");
-				if (chan->_state != AST_STATE_UP)
+				if (ast_channel_state(chan) != AST_STATE_UP)
 					ast_answer(chan);
-				res = ast_streamfile(chan, cachefile, chan->language);
+				res = ast_streamfile(chan, cachefile, ast_channel_language(chan));
 				if (res) {
 					ast_log(LOG_ERROR, "eSpeak: ast_streamfile from cache failed on %s\n",
-							chan->name);
+							ast_channel_name(chan));
 				} else {
 					res = ast_waitstream(chan, args.interrupt);
 					ast_stopstream(chan);
@@ -232,9 +250,7 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 	}
 
 	/* Invoke eSpeak */
-	sample_rate = espeak_Initialize(AUDIO_OUTPUT_SYNCHRONOUS, ESPK_BUFFER, NULL, 0);
-
-	if (sample_rate == -1) {
+	if ((sample_rate = espeak_Initialize(AUDIO_OUTPUT_SYNCHRONOUS, ESPK_BUFFER, NULL, 0)) == -1) {
 		ast_log(LOG_ERROR, "eSpeak: Internal espeak error, aborting.\n");
 		return -1;
 	}
@@ -246,22 +262,17 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 	espeak_SetParameter(espeakPITCH, pitch, 0);
 	espeak_SetParameter(espeakCAPITALS, capind, 0);
 
-	raw_fd = mkstemp(raw_name);
-
-	if (raw_fd == -1) {
+	if ((raw_fd = mkstemp(raw_name)) == -1) {
 		ast_log(LOG_ERROR, "eSpeak: Failed to create audio file.\n");
 		return -1;
 	}
-
-	fl = fdopen(raw_fd, "w+");
-	if (fl == NULL) {
+	if ((fl = fdopen(raw_fd, "w+")) == NULL) {
 		ast_log(LOG_ERROR, "eSpeak: Failed to open audio file '%s'\n", raw_name);
 		return -1;
 	}
 
-	espk_error =
-		espeak_Synth(args.text, strlen(args.text), 0, POS_CHARACTER,
-					 (int) strlen(args.text), espeakCHARS_AUTO, NULL, fl);
+	espk_error = espeak_Synth(args.text, strlen(args.text), 0, POS_CHARACTER,
+			(int) strlen(args.text), espeakCHARS_AUTO, NULL, fl);
 	espeak_Terminate();
 	fclose(fl);
 	if (espk_error != EE_OK) {
@@ -277,18 +288,24 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 		src_info.samplerate = (int) sample_rate;
 		src_info.channels = 1;
 		src_info.format = RAW_PCM_S16LE;
-		src_file = sf_open(raw_name, SFM_RDWR, &src_info);
-		if (src_file == NULL) {
+		if ((src_file = sf_open(raw_name, SFM_RDWR, &src_info)) == NULL) {
 			ast_log(LOG_ERROR, "eSpeak: Failed to read raw audio data '%s'\n", raw_name);
 			ast_filedelete(raw_name, NULL);
 			return -1;
 		}
-		src_buff = (float *) ast_malloc(src_info.frames * sizeof(float));
+		if ((src_buff = (float *) ast_malloc(src_info.frames * sizeof(float))) == NULL) {
+			ast_log(LOG_ERROR, "eSpeak: Failed to allocate memory for resampling.\n");
+			sf_close(src_file);
+			return -1;
+		}
 		sf_readf_float(src_file, src_buff, src_info.frames);
-		dst_frames =
-			src_info.frames * (sf_count_t) target_sample_rate / (sf_count_t) sample_rate;
-		dst_buff = (float *) ast_malloc(dst_frames * sizeof(float));
-
+		dst_frames = src_info.frames * (sf_count_t) target_sample_rate / (sf_count_t) sample_rate;
+		if ((dst_buff = (float *) ast_malloc(dst_frames * sizeof(float))) == NULL) {
+			ast_log(LOG_ERROR, "eSpeak: Failed to allocate memory for resampling.\n");
+			sf_close(src_file);
+			ast_free(src_buff);
+			return -1;
+		}
 		rate_change.data_in = src_buff;
 		rate_change.data_out = dst_buff;
 		rate_change.input_frames = src_info.frames;
@@ -328,11 +345,11 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 		ast_filecopy(raw_name, cachefile, NULL);
 	}
 
-	if (chan->_state != AST_STATE_UP)
+	if (ast_channel_state(chan) != AST_STATE_UP)
 		ast_answer(chan);
-	res = ast_streamfile(chan, raw_name, chan->language);
+	res = ast_streamfile(chan, raw_name, ast_channel_language(chan));
 	if (res) {
-		ast_log(LOG_ERROR, "eSpeak: ast_streamfile failed on %s\n", chan->name);
+		ast_log(LOG_ERROR, "eSpeak: ast_streamfile failed on %s\n", ast_channel_name(chan));
 	} else {
 		res = ast_waitstream(chan, args.interrupt);
 		ast_stopstream(chan);
@@ -342,10 +359,10 @@ static int espeak_exec(struct ast_channel *chan, const char *data)
 	return res;
 }
 
-static int reload(void)
+static int reload_module(void)
 {
 	ast_config_destroy(cfg);
-	read_config();
+	read_config(ESPEAK_CONFIG);
 	return 0;
 }
 
@@ -357,13 +374,13 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	read_config();
+	read_config(ESPEAK_CONFIG);
 	return ast_register_application(app, espeak_exec, synopsis, descrip) ?
 		AST_MODULE_LOAD_DECLINE : AST_MODULE_LOAD_SUCCESS;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "eSpeak TTS Interface",
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-			);
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+);
